/***************************************************************************
 *   Copyright (C) 2007 by Andreas Loibl   *
 *   andreas@andreas-loibl.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <kglobalsettings.h>
#include <qlayout.h>

#include "acrilistbox.h"

AcriListBox::AcriListBox(QWidget* parent, const char* name)
    : QScrollView(parent, name, WStaticContents),
    m_selectedAcriList(0)
{
    this->setResizePolicy(QScrollView::AutoOneFit);
    
    this->acrilistBox = new QWidget(this->viewport());
    acrilistBox->setPaletteBackgroundColor(KGlobalSettings::baseColor());
    this->addChild(this->acrilistBox, 0, 0);
    this->acrilistBox->show();
    
    this->layout = new QVBoxLayout(this->acrilistBox);
    this->layout->addStretch(1);
    this->layout->setMargin(0);
}

void AcriListBox::insertItem( AcriListInfo info )
{
    int i = this->m_acrilistWidgetList.count();
    bool odd = ! (i & 1);
    AcriListWidget *itemWidget = new AcriListWidget(info, odd, this->acrilistBox);
    itemWidget->show();
    this->layout->insertWidget(i, itemWidget);
    this->m_acrilistWidgetList.append(itemWidget);
    this->m_acrilists.append(info);
    
    connect(itemWidget, SIGNAL(clicked(AcriListWidget*)),
                this, SLOT(selectAcriList(AcriListWidget*)));
    connect(itemWidget, SIGNAL(doubleClicked(AcriListWidget*)),
                this, SLOT(doubleClickedItem(AcriListWidget*)));
}

void AcriListBox::clear()
{
    for (QValueList<AcriListWidget*>::iterator it = m_acrilistWidgetList.begin();
             it != m_acrilistWidgetList.end();
             ++it)
    {
            delete *it;
    }
    this->m_acrilistWidgetList.clear();
    this->m_acrilists.clear();
    m_selectedAcriList = 0;
}

void AcriListBox::selectAcriList(AcriListWidget *acrilist)
{
    if (m_selectedAcriList)
    {
        m_selectedAcriList->setSelected(false);
    }

    m_selectedAcriList = acrilist;

    if (m_selectedAcriList)
    {
        m_selectedAcriList->setSelected(true);
        emit selectionChanged();
        emit selected( acrilist );
        emit selected( &this->m_acrilists.at(this->m_acrilistWidgetList.findIndex(acrilist)) );
        emit selected( this->m_acrilistWidgetList.findIndex(acrilist) );
    }
}

void AcriListBox::doubleClickedItem(AcriListWidget *acrilist)
{
    emit doubleClicked( acrilist );
}

AcriListInfo AcriListBox::currentItem()
{
    int index = this->m_acrilistWidgetList.findIndex(m_selectedAcriList);
    AcriListInfo info = this->m_acrilists.at(index);
    return(info);
}

int AcriListBox::count()
{
    return(this->m_acrilistWidgetList.count());
}

void AcriListBox::setCurrentItem(int index)
{
    selectAcriList(*this->m_acrilistWidgetList.at(index));
}