/***************************************************************************
 *   Copyright (C) 2007 by Andreas Loibl   *
 *   andreas@andreas-loibl.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kapplication.h>
#include <kcmdlineargs.h>
#include <kiconloader.h>
#include <klistbox.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kprogress.h>
#include <kpassdlg.h>
#include <ktempfile.h>

#include <qapplication.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qcursor.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qregexp.h>
#include <qscrollview.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qstringlist.h>
#include <qtable.h>
#include <qtabwidget.h>
#include <qtextbrowser.h>
#include <qtextstream.h>
#include <qtoolbutton.h>
#include <qtimer.h>
#include <qtooltip.h>
#include <qvalidator.h>
#include <qvbox.h>

#include "acritoxinstaller.h"
#include "acrilistinfo.h"
#include "acrilistbox.h"
#include "dirselector.h"

AcritoxInstaller::AcritoxInstaller(QWidget* parent, const char* name, bool modal, WFlags fl)
    : MainWizard(parent,name, modal,fl)
{
    connect(this, SIGNAL(helpClicked()), this, SLOT(slotHelp()));
    
    backend = new Backend();
    backend->runBackend();
    connect(backend, SIGNAL(receivedError(uint, QString)),
            this, SLOT(handleBackendError(uint, QString)));
    
    setNextEnabled(wpDisclaimer, false);
    setNextEnabled(wpConfirmPartitioning, false);
    setNextEnabled(wpConfirmInstallation, false);
    setNextEnabled(wpSwap, false);
    setNextEnabled(wpRoot, false);
    setNextEnabled(wpUser, false);
    
    setBackEnabled(wpInstallation, false);
    setBackEnabled(wpSwap, false);
    
    partitionQueryInputFrame->hide();
    
    rootPassword1->setMaxPasswordLength(20);
    rootPassword2->setMaxPasswordLength(20);
    userPassword1->setMaxPasswordLength(20);
    userPassword2->setMaxPasswordLength(20);
    
    backend->wait();
    backend->exec("kde_medianotifier off"); backend->wait();
    backend->exec("is_ubuntu"); backend->wait();
    isUbuntu = false;
    if(backend->getOutput().contains("ubuntu")) isUbuntu = true;
    
    hdMapTable->setColumnReadOnly(0, true);
    hdMapTable->setColumnStretchable(0, true);
    hdMapTable->setColumnStretchable(1, true);
    hdMapTable->setColumnStretchable(2, true);
    hdMapTable->setColumnWidth(3, 80);
    
    progressDlg = 0;
    statusBusy = false;
    
    // Input Validators
    QValidator* usernameValidator = new QRegExpValidator(QRegExp("[a-zA-Z0-9-_.]*"), this);
    username->setValidator( usernameValidator );
    QValidator* hostnameValidator = new QRegExpValidator(QRegExp("[a-zA-Z0-9][a-zA-Z0-9.-]*[a-zA-Z0-9]"), this);
    hostname->setValidator( hostnameValidator );
    
    // ToolButton-Icons
    deviceMapUp->setIconSet( SmallIcon("up") );
    deviceMapDown->setIconSet( SmallIcon("down") );
    
    // fill installTypeList
    installTypeList->clear();
    installTypeList->insertItem(AcriListInfo::AcriListInfo(1,i18n("Install"),i18n("Fresh install to your Harddisk"),"acritoxinstaller"));
    //installTypeList->insertItem(AcriListInfo::AcriListInfo(2,i18n("Update Install"),i18n("Update an existing installation"),"acritoxinstaller"));
    installTypeList->insertItem(AcriListInfo::AcriListInfo(3,i18n("Recover"),i18n("Reinstall bootloader of an existing installation"),"acritoxinstaller"));
    installTypeList->setCurrentItem(0);

    handleCommandLine();
    QTimer::singleShot( 20, this, SLOT(back()) );
}

AcritoxInstaller::~AcritoxInstaller()
{}

void AcritoxInstaller::handleCommandLine()
{
    KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
    if(!args->isSet("raid")) backend->exec("set_flag noraid");
}

void AcritoxInstaller::setStatusBusy(bool busy)
{
    statusBusy = busy;
    cancelButton()->setDisabled(busy);
}

void AcritoxInstaller::reject()
{
    if( statusBusy || ( !statusFinished && KMessageBox::warningYesNoCancel(this,
                            i18n("Do you really want to quit?"),
                            i18n("Quit AcritoxInstaller")) != KMessageBox::Yes ) )
    {
        return;
    }
    
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    
    backend->wait();
    backend->exec("cleanup");
    backend->exec("kde_medianotifier on");
    backend->wait();
    
    QWizard::reject();
}

void AcritoxInstaller::slotHelp()
{
    kapp->invokeHelp( QString( "acritoxinstaller" ) );
}

/*$SPECIALIZATION$*/
void AcritoxInstaller::back()
{
  /********************************** Partitions Query **********************************/
  if(this->currentPage() == wpPartitionsQuery)
  {
    if(backend->isQuery())
    {
      backend->answerQuery("");
    }
    disconnect(backend, SIGNAL(receivingStdout()),
            this, SLOT(prepareLogReceivedOutput()));
    partitionQueryInputFrame->hide();
    partitionsQueryLog->setText("");
  }
  
  /********************************** Confirm Automatic Partitioning **********************************/
  else if(this->currentPage() == wpConfirmPartitioning)
  {
    confirmCheckBox->setChecked(false);
    disconnect(backend, SIGNAL(receivingStdout()),
            this, SLOT(runLogReceivedOutput()));
    partitionsRunLog->setText("");
  }
  
  /********************************** Partition map **********************************/
  else if(this->currentPage() == wpHdMap)
  {
    QString tmpHdMap;
    for ( int j = 0; j < hdMapTable->numRows(); j++ )
    {
        if ( hdMapTable->text( j, 1 ).length() )
        {
            tmpHdMap.append(hdMapTable->text( j, 0 ) + ":");
            tmpHdMap.append(hdMapTable->text( j, 1 ) + ":");
            tmpHdMap.append(hdMapTable->text( j, 2 ) + ":");
            QTableItem* item = hdMapTable->item(j, 3);
            QCheckTableItem *citem = dynamic_cast<QCheckTableItem * > (item);
            if( citem->isChecked() )
                tmpHdMap.append("auto ");
            else
                tmpHdMap.append("noauto ");
        }
    }
    
    backend->exec("set_config hd_map_nocheck '"+tmpHdMap.replace("'","")+"'");
  }
  
  backend->wait();
  if(backend->errorOccurred()) return;
  QWizard::back();
  this->pageChanged();
}

void AcritoxInstaller::next()
{
  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
  if(this->currentPage() == wpInstallType)
  {
    setAppropriate(wpPartitions, false);
    setAppropriate(wpPartitionsQuery, false);
    setAppropriate(wpSelectPartitionManager, false);
    setAppropriate(wpWaitForPartitioning, false);
    setAppropriate(wpConfirmPartitioning, false);
    setAppropriate(wpSwap, false);
    setAppropriate(wpSelectRoot, false);
    setAppropriate(wpUpdateInstallExcludeList, false);
    setAppropriate(wpRootPartition, false);
    setAppropriate(wpHdMap, false);
    setAppropriate(wpBootloader, false);
    setAppropriate(wpBootOrder, false);
    setAppropriate(wpRoot, false);
    setAppropriate(wpUser, false);
    setAppropriate(wpNetwork, false);
    setAppropriate(wpConfirmInstallation, false);
    setAppropriate(wpInstallation, false);

    switch(installTypeList->currentItem().id())
    {
    case 1:
      setAppropriate(wpPartitions, true);
      setAppropriate(wpPartitionsQuery, true);
      setAppropriate(wpSelectPartitionManager, true);
      setAppropriate(wpWaitForPartitioning, true);
      setAppropriate(wpConfirmPartitioning, true);
      setAppropriate(wpSwap, true);
      setAppropriate(wpRootPartition, true);
      setAppropriate(wpHdMap, true);
      setAppropriate(wpBootloader, true);
      setAppropriate(wpBootOrder, true);
      setAppropriate(wpRoot, true);
      setAppropriate(wpUser, true);
      setAppropriate(wpNetwork, true);
      setAppropriate(wpConfirmInstallation, true);
      setAppropriate(wpInstallation, true);
      break;
    case 2:
      setAppropriate(wpSelectRoot, true);
      setAppropriate(wpUpdateInstallExcludeList, true);
      setAppropriate(wpBootloader, true);
      setAppropriate(wpBootOrder, true);
      setAppropriate(wpInstallation, true);
      break;
    case 3:
      setAppropriate(wpSelectRoot, true);
      setAppropriate(wpBootloader, true);
      setAppropriate(wpBootOrder, true);
      break;
    }
  }
  
  /********************************** Partitions **********************************/
  else if(this->currentPage() == wpPartitions)
  {
    setAppropriate(wpSelectPartitionManager, false);
    setAppropriate(wpWaitForPartitioning, false);
    setAppropriate(wpPartitionsQuery, false);
    setAppropriate(wpConfirmPartitioning, false);
    
    setAppropriate(wpRootPartition, false);
    setAppropriate(wpHdMap, true);
    setBackEnabled(wpHdMap, false);
    
    setAppropriate(wpRoot, !isUbuntu); // show root-password wizard page only if not an ubuntu-system
    setAppropriate(wpUser, true);
    setAppropriate(wpNetwork, true);
    setAppropriate(wpConfirmInstallation, true);
    
    if(installOptionsList->currentItem().id() == -1) // Manual partitioning
    {
      setAppropriate(wpSelectPartitionManager, true);
      setAppropriate(wpWaitForPartitioning, true);
      setAppropriate(wpRootPartition, true);
    }
    else
    {
      setAppropriate(wpPartitionsQuery, true);
      setAppropriate(wpConfirmPartitioning, true);
      setAppropriate(wpHdMap, true);
    }
  }
  
  /********************************** Partitions Query **********************************/
  else if(this->currentPage() == wpPartitionsQuery)
  {
    if(backend->isQuery() || partitionQueryInputFrame->isVisible())
    {
      QApplication::restoreOverrideCursor();
      this->pageChanged();
      return;
    }
    else
    {
      setAppropriate(wpPartitionsQuery, false);
      disconnect(backend, SIGNAL(receivingStdout()),
            this, SLOT(prepareLogReceivedOutput()));
      partitionsQueryLog->setText("");
      QApplication::restoreOverrideCursor();
    }
  }
  
  /********************************** Select Partition Manager **********************************/
  else if(this->currentPage() == wpSelectPartitionManager)
  {
    if(partitionManagerList->currentItem().id() != -1)
    {
      QString partitionManager = partitionManagerList->currentItem().name();
      if(partitionManager) backend->exec("run_partition_manager "+partitionManager);
    }
  }
  
  /********************************** Wait for manual Partitioning **********************************/
  else if(this->currentPage() == wpWaitForPartitioning)
  {
    
  }
  
  /********************************** Confirm Automatic Partitioning **********************************/
  else if(this->currentPage() == wpConfirmPartitioning)
  {
    if(confirmCheckBox->isChecked())
    {
      if(confirmCheckBox->isEnabled())
      {
        confirmCheckBox->setEnabled(false);
        this->pageChanged();
        return;
      }
      else
      {
        disconnect(backend, SIGNAL(receivingStdout()),
                this, SLOT(runLogReceivedOutput()));
        partitionsRunLog->setText("");
        setStatusBusy(false);
        QApplication::restoreOverrideCursor();
      }
    }
  }
  
  /********************************** Swap **********************************/
  else if(this->currentPage() == wpSwap)
  {
    progressDlg = new KProgressDialog( 0, 0, i18n("Applying swap adjustments"),
                      i18n("Please wait... Applying adjustments..."), true );
    progressDlg->setAllowCancel( false );
    progressDlg->setAutoClose( false );
    progressDlg->progressBar()->setTotalSteps(swapAdjustmentsList->count());
    progressDlg->show();
    
    for ( QStringList::Iterator it = adjustmentsList.begin(); it != adjustmentsList.end(); ++it ) {
      backend->exec(*it);
      progressDlg->progressBar()->setProgress(progressDlg->progressBar()->progress()+1);
    }
  }
  
  /********************************** Recover / Update Install **********************************/
  else if(this->currentPage() == wpSelectRoot)
  {
    progressDlg = new KProgressDialog( 0, 0, i18n("Mounting partitions..."),
                      i18n("Please wait... Mounting partitions..."), true );
    progressDlg->setAllowCancel( false );
    progressDlg->setAutoClose( false );
    connect( backend, SIGNAL(receivedProgress(int)), progressDlg->progressBar(), SLOT(setProgress(int)) );
    progressDlg->show();

    switch(installTypeList->currentItem().id())
    {
    case 2:
      backend->exec("update_install_set root '"+updateRoot->currentText().section(" ",0,0)+"'");
      backend->exec("update_install_prepare");
      backend->exec("update_install_init_lists");
      break;
    case 3:
      backend->exec("set_config hd_map_nocheck '"+updateRoot->currentText().section(" ",0,0)+":/::auto'");
      backend->exec("recover_prepare");
      break;
    }
  }
  
  /********************************** Update Install Exclude List **********************************/
  else if(this->currentPage() == wpUpdateInstallExcludeList)
  {
    KTempFile temp;
    
    QTextStream* stream = temp.textStream();
    QStringList lines = excludeList->directories();
    for ( QStringList::Iterator it = lines.begin(); it != lines.end(); ++it )
        *stream << *it << "\n";
    temp.close();
    backend->exec("update_install_set unaffected_list '"+ temp.name() +"'");
    temp.unlink();
//       backend->exec("update_install_prepare");
  }
  
  /********************************** Root Partition **********************************/
  else if(this->currentPage() == wpRootPartition)
  {
    QString tmpHdMap;
    
    tmpHdMap+=installRoot->currentText().section(" ",0,0);
    tmpHdMap+=":/:";
    if(installRootFormatCheckbox->isChecked()) tmpHdMap+=installRootFormat->currentText().section(" ",0,0);
    tmpHdMap+=":auto";
    
    if(hdMapMode->currentPage() == tabSimpleHdMap)
    {
        backend->exec("set_config hd_map_root '"+tmpHdMap+"'");
        progressDlg = new KProgressDialog( 0, 0, i18n("Checking hd_map..."),
                          i18n("Please wait... Checking hd_map..."), true );
        progressDlg->setAllowCancel( false );
        connect( backend, SIGNAL(receivedProgress(int)), progressDlg->progressBar(), SLOT(setProgress(int)) );
        progressDlg->show();
        backend->exec("check_partitions_for_install");
        setAppropriate(wpHdMap, false);
    }
    else
    {
        // only set hd_map if a root partition was selected
        if(installRoot->count()) backend->exec("set_config hd_map_root '"+tmpHdMap+"'");
        setAppropriate(wpHdMap, true);
        setBackEnabled(wpHdMap, true);
    }
  }
  
  /********************************** Partition map **********************************/
  else if(this->currentPage() == wpHdMap)
  {
    QString tmpHdMap;
    for ( int j = 0; j < hdMapTable->numRows(); j++ )
    {
        if ( hdMapTable->text( j, 1 ).length() )
        {
            tmpHdMap.append(hdMapTable->text( j, 0 ) + ":");
            tmpHdMap.append(hdMapTable->text( j, 1 ) + ":");
            tmpHdMap.append(hdMapTable->text( j, 2 ) + ":");
            QTableItem* item = hdMapTable->item(j, 3);
            QCheckTableItem *citem = dynamic_cast<QCheckTableItem * > (item);
            if( citem->isChecked() )
                tmpHdMap.append("auto ");
            else
                tmpHdMap.append("noauto ");
        }
    }
    
    progressDlg = new KProgressDialog( 0, 0, i18n("Checking hd_map..."),
                      i18n("Please wait... Checking hd_map..."), true );
    progressDlg->setAllowCancel( false );
    connect( backend, SIGNAL(receivedProgress(int)), progressDlg->progressBar(), SLOT(setProgress(int)) );
    progressDlg->show();
    backend->exec("set_config hd_map '"+tmpHdMap.replace("'","")+"'");
  }
  
  /********************************** Bootloader **********************************/
  else if(this->currentPage() == wpBootloader)
  {
    if(bootloader->currentItem() == 0)
      backend->exec("set_config bootloader grub");
    else if(bootloader->currentItem() == 1)
      backend->exec("set_config bootloader lilo");
    
    if(bootDisk->isChecked())
      backend->exec("set_config bootdisk 1");
    else
      backend->exec("set_config bootdisk 0");
    
    backend->exec("set_config bootloadertarget "+bootloaderTarget->currentText().section(" ",1,1));
    
    setAppropriate(wpBootOrder, bootExpertOptions->isChecked());
  }
  
  /********************************** Device Map **********************************/
  else if(this->currentPage() == wpBootOrder)
  {
    backend->exec("write_device_map", false);
    for (int i = 0; i < deviceMapNumber->count(); i++)
    {
        backend->writeStdin(deviceMapNumber->text(i) + "\t" + deviceMapDevice->text(i).section(" ",0,0));
    }
    backend->writeStdin("");
  }
  
  /********************************** Root-account configuration **********************************/
  else if(this->currentPage() == wpRoot)
  {
    QString rootPW = rootPassword1->password();
    backend->exec("set_config rootpwd " + encrypt(rootPW));
  }
  
  /********************************** User configuration **********************************/
  else if(this->currentPage() == wpUser)
  {
    QString userPW = userPassword1->password();
    backend->exec("set_config userpwd " + encrypt(userPW));
    backend->exec("set_config username " + username->text());
    backend->exec("set_config realname " + realname->text());
    
    if(kdmAutologin->isChecked())
      backend->exec("set_config autologin yes");
    else
      backend->exec("set_config autologin no");
  }
  
  /********************************** User configuration **********************************/
  else if(this->currentPage() == wpNetwork)
  {
    backend->exec("set_config hostname " + hostname->text());
  }
  
  backend->wait();
  QApplication::restoreOverrideCursor();
  progressEnd();
  
  if(backend->errorOccurred()) return;
  QWizard::next();
  this->pageChanged();
}

void AcritoxInstaller::pageChanged()
{
  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
  /********************************** Disclaimer **********************************/
  if(this->currentPage() == wpDisclaimer)
  {
    backend->exec("init_filesystems");
    backend->wait();
    connect(backend, SIGNAL(receivedProgress(int)),
            progressAnalyze, SLOT(setProgress(int)));
    backend->exec("analyze_disks");
    backend->wait();
    disconnect(backend, SIGNAL(receivedProgress(int)),
            progressAnalyze, SLOT(setProgress(int)));
    progressAnalyze->setHidden(true);
    lblAnalyzePatient->setHidden(true);
    setBackEnabled(wpPartitions, false);
    setNextEnabled(wpDisclaimer, true);
  }
  
  /********************************** Partitions **********************************/
  else if(this->currentPage() == wpPartitions)
  {
    if(installOptionsList->count()<2)
    {
      installOptionsList->clear();
      installOptionsList->insertItem(AcriListInfo::AcriListInfo(-1,i18n("Manual partitioning"),i18n("You will have to do the partitioning with a partition-manager manually. This way you have more possibilities to adjust the installation to your needs. The automatic partitioning lacks several features and is only a (computer generated) list of some possible options."),"run"));
      installOptionsList->setCurrentItem(0);

      backend->exec("show_install_options");
      backend->wait();
      
      QStringList interfacesList = QStringList::split( "---", backend->getOutput() );
      for ( QStringList::Iterator it = interfacesList.begin(); it != interfacesList.end(); ++it ) {
        QString content = (*it);
        QString idstr = content.section("\n",0,0).section(" - ",0,0); int id = idstr.toInt();
        QString title = content.section("\n",0,0).section(" - ",1);
        QString description = content.section("\n",1);
        installOptionsList->insertItem(AcriListInfo::AcriListInfo(id,title,description,"wizard"));
      }
    }
    setAppropriate(wpPartitionsQuery, true);
  }
  
  /********************************** Partitions Query **********************************/
  else if(this->currentPage() == wpPartitionsQuery)
  {
    if(partitionQueryInputFrame->isVisible())
    {
          backend->answerQuery( QString::number(partitionQuerySpin->value()) );
          partitionQueryInputFrame->hide();
          next();
    }
    else if(backend->isQuery())
    {
      QString query = backend->getQuery().stripWhiteSpace();
      
      QString headline;
      
      QString device = backend->data("device");
      QString volg = backend->data("volg");
      QString action = backend->data("action");
      QString target = backend->data("target");
      
      if(action == "create")
      {
        if(target == "root")
          headline = i18n("Create install partition on %1").arg(device);
        else if(target == "boot")
          headline = i18n("Create boot partition on %1").arg(device);
        else if(target == "lvm-vg")
          headline = i18n("Create LVM-Volumegroup '%1' in a new partition on %2").arg(volg).arg(device);
      }
      else if(action == "resize")
      {
        headline = i18n("Resize partition on %1").arg(device);
      }
      
      partitionQueryIcon->setPixmap(DesktopIcon("hdd_unmount"));
      partitionQueryDiskInfo->setText(i18n("<h3>%1</h3><p>Please adjust partition size (Megabytes)</p>").arg(headline));
      
      if( query.section(" ",0,0) == "partitionrange" )
      {
        int min = query.section(" ",1,1).toInt();
        int max = query.section(" ",2,2).toInt();
        int def = query.section(" ",3,3).toInt();
        
        partitionQuerySlider->setMinValue(min);
        partitionQuerySlider->setMaxValue(max);
        partitionQuerySpin->setMinValue(min);
        partitionQuerySpin->setMaxValue(max);
        
        partitionQuerySpin->setValue(def);
      }
      QApplication::restoreOverrideCursor(); // No wait-cursor while asking the user for input
      partitionQueryInputFrame->show();
    }
    else 
    {
      connect(backend, SIGNAL(receivingStdout()),
            this, SLOT(prepareLogReceivedOutput()));
      setStatusBusy(true); // Don't allow the user to close the window now...
      backend->exec("select_install_option " + QString::number(installOptionsList->currentItem().id()));
      backend->exec("selected_install_option prepare");
      next();
    }
  }
  
  /********************************** Confirm Automatic Partitioning **********************************/
  else if(this->currentPage() == wpConfirmPartitioning)
  {
    if(confirmCheckBox->isChecked())
    {
      setStatusBusy(true); // Don't allow the user to close the window now...
      partitionsRunLog->setText("");
      backend->exec("selected_install_option process");
      QApplication::restoreOverrideCursor(); // No wait-cursor after backend has finished
      partitionsRunLog->append(i18n("<br><br><p><b>Press &raquo;Next&laquo; to continue.</b></p>"));
    }
    else
    {
      connect(backend, SIGNAL(receivingStdout()),
            this, SLOT(runLogReceivedOutput()));
    }
  }
  
  /********************************** Select Partition Manager **********************************/
  else if(this->currentPage() == wpSelectPartitionManager)
  {
    if(partitionManagerList->count()<2)
    {
      partitionManagerList->clear();
      partitionManagerList->insertItem(AcriListInfo::AcriListInfo(-1,i18n("already partitioned"),i18n("If you have already partitioned for the installation and don't want to start a partition manager then select this."),"button_ok"));
      
      backend->exec("list_partition_managers");
      backend->wait();
      
      QStringList partManList = QStringList::split( "\n", backend->getOutput() );
      for ( QStringList::Iterator it = partManList.begin(); it != partManList.end(); ++it ) {
        QString content = (*it);
        QString idstr = content.section(":",0,0); int id = idstr.toInt();
        QString title = content.section(":",1);
        ++it;
        QString description = (*it);
        partitionManagerList->insertItem(AcriListInfo::AcriListInfo(id,title,description,"run"));
      }
      if(partitionManagerList->count()) partitionManagerList->setCurrentItem(0);
    }
  }
  
  /********************************** Wait for manual Partitioning **********************************/
  else if(this->currentPage() == wpWaitForPartitioning)
  {
    
  }
  
  /********************************** Swap **********************************/
  else if(this->currentPage() == wpSwap)
  {
    swapAdjustmentsList->clear();
    confirmSwapAdjustments->setEnabled(true);
    confirmSwapAdjustments->setChecked(false);
    setNextEnabled(wpSwap, false);
    
    backend->exec("fix_swap_partitions");
    backend->wait();
    
    adjustmentsList = QStringList::split( "\n", backend->getOutput() );
    int id=0;
    for ( QStringList::Iterator it = adjustmentsList.begin(); it != adjustmentsList.end(); ++it ) {
      QString command = QString(*it).section(" ", 0, 0);
      QString device = QString(*it).section(" ", 1, 1);
      QString title = (*it);
      QString description = "";
      if(command == "mkswap") description=i18n("Set up a Linux swap area on %1 (everything on this partition will be deleted!)").arg(device);
      else if(command == "fdisk_chtype_swap") description=i18n("Change type of partition %1 to Linux swap (82)").arg(device);
      else if(command == "add_swap_to_fstab") description=i18n("Add a fstab-entry for swap-partition %1").arg(device);
      else if(command == "swapon") description=i18n("Enables swapping on %1").arg(device);
      swapAdjustmentsList->insertItem(AcriListInfo::AcriListInfo(id++,title,description,"run"));
    }
    if(swapAdjustmentsList->count() == 0)
    {
      swapAdjustmentsList->insertItem(AcriListInfo::AcriListInfo(0,i18n("No adjustments needed"),i18n("Press <b>Next</b> to continue"),"button_ok"));
      confirmSwapAdjustments->setChecked(true);
      confirmSwapAdjustments->setEnabled(false);
      setNextEnabled(wpSwap, true);
    }
  }
  
  /********************************** Recover / Update Install **********************************/
  else if(this->currentPage() == wpSelectRoot)
  {
    switch(installTypeList->currentItem().id())
    {
    case 2:
      backend->exec("list_possible_update_partitions");
      break;
    case 3:
      backend->exec("list_possible_root_partitions");
      break;
    }
    
    backend->wait();
    QStringList interfacesList = QStringList::split( "\n", backend->getOutput() );
    QString oldUpdateRoot;
    if(updateRoot->count())
        oldUpdateRoot=updateRoot->currentText().section(" ",0,0);
    
    updateRoot->clear();
    for ( QStringList::Iterator it = interfacesList.begin(); it != interfacesList.end(); ++it ) {
        QString line = *it;
        QString device = line.section(":",0,0);
        QString size = line.section(":",1,1);
        size = this->formatSizeLong(size.toLongLong());
        updateRoot->insertItem( DesktopIcon("hdd_unmount"), QString("%1 (%2)").arg(device).arg(size));
    }
    updateRoot->setCurrentItem(0);
    for( int i = 0; i < updateRoot->count(); i++)
    {
       if(updateRoot->text(i).section(" ",0,0) != oldUpdateRoot) continue;
       updateRoot->setCurrentItem(i);
       break;
    }
  }
  
  /********************************** Update Install Exclude List **********************************/
  else if(this->currentPage() == wpUpdateInstallExcludeList)
  {
//     excludeList = new DirectoryList(wpUpdateInstallExcludeListLayout, "excludeList", "/media/hdinstall");
       excludeList->setTopDir("/media/hdinstall");
       
       excludeList->pathsDisabled.append("/media/hdinstall/etc");
       
       backend->exec("update_install_get deletion_list");
       excludeList->pathsRDisabled = QStringList::split( "\n", backend->getOutput() );
       
//     excludeList->m_dirs_recursive_disabled.append("/media/hdinstall/sys");
//     wpUpdateInstallExcludeListLayout->addWidget(excludeList,0,0);
//     excludeList->init("/media/hdinstall");
       
       backend->exec("update_install_get unaffected_list");
       excludeList->setDirectories(QStringList::split( "\n", backend->getOutput() ));
       
       excludeList->refresh();
  }
  
  /********************************** Root Partition **********************************/
  else if(this->currentPage() == wpRootPartition)
  {
      backend->exec("list_possible_root_partitions");
      backend->wait();
      
      QStringList interfacesList = QStringList::split( "\n", backend->getOutput() );
      
      installRoot->clear();
      for ( QStringList::Iterator it = interfacesList.begin(); it != interfacesList.end(); ++it ) {
          QString line = *it;
          QString device = line.section(":",0,0);
          QString size = line.section(":",1,1);
          size = this->formatSizeLong(size.toLongLong());
          installRoot->insertItem( DesktopIcon("hdd_unmount"), QString("%1 (%2)").arg(device).arg(size));
//           installRoot->insertItem( DesktopIcon("hdd_unmount"), QString("%1 ").arg(line));
      }
      installRoot->setCurrentItem(0);
      
      backend->exec("get_config root_dev");
      QString oldRootPartition = backend->getOutput().stripWhiteSpace();
      for( int i = 0; i < installRoot->count(); i++)
      {
         if(installRoot->text(i).section(" ",0,0) != oldRootPartition) continue;
         installRoot->setCurrentItem(i);
         break;
      }
      
      backend->exec("get_config root_fs");
      if(backend->getOutput().stripWhiteSpace().length())
      {
          for (int i = 0; i < installRootFormat->count(); i++)
          {
              if(installRootFormat->text(i) == backend->getOutput().stripWhiteSpace())
              {
                  installRootFormat->setCurrentItem(i);
                  break;
              }
          }
          installRootFormatCheckbox->setChecked(true);
      }
      else
      {
          installRootFormat->setCurrentItem(0);
          installRootFormatCheckbox->setChecked(false);
      }
  }
  
  /********************************** Partition map **********************************/
  else if(this->currentPage() == wpHdMap)
  {
      backend->exec("get_full_hd_map");
      backend->wait();
      
      QStringList hdMapList = QStringList::split( "\n", backend->getOutput() );
//       MAP->clear();
      QString hdMap;
      QStringList filesystems;
      filesystems << "" << "reiserfs" << "ext3" << "ext4" << "jfs";
      
      hdMapTable->setNumRows( 0 );
      for ( QStringList::Iterator it = hdMapList.begin(); it != hdMapList.end(); ++it ) {
          QString line = *it;
          QString device = line.section(":",0,0);
          QString mountpoint = line.section(":",1,1);
          QString filesystem = line.section(":",2,2);
          QString automount = line.section(":",3,3);
          hdMap.append(line+" ");
//           MAP->insertItem( device, mountpoint, filesystem);
          QComboTableItem* formatFs = new QComboTableItem(hdMapTable, filesystems);
          QCheckTableItem* autoMount = new QCheckTableItem(hdMapTable, "auto");
          formatFs->setCurrentItem(filesystem);
          if(automount == "auto") autoMount->setChecked(true);
          int newRow = hdMapTable->numRows();
          hdMapTable->insertRows( newRow );
          hdMapTable->setPixmap(newRow,0,SmallIcon("hdd_unmount"));
          hdMapTable->setText(newRow,0,device);
          hdMapTable->setText(newRow,1,mountpoint);
          // hdMapTable->setText(tempRow,2,"ext3");
          hdMapTable->setItem(newRow,2,formatFs);
          hdMapTable->setItem(newRow,3,autoMount);
      }
//       tmpHdMap->setText(hdMap.stripWhiteSpace());
  }
  
  /********************************** Bootloader **********************************/
  else if(this->currentPage() == wpBootloader)
  {
      backend->exec("list_bootloader_targets");
      backend->wait();
      
      QStringList interfacesList = QStringList::split( "\n", backend->getOutput() );
      QString oldBootloaderTarget;
      if(bootloaderTarget->count())
          oldBootloaderTarget=bootloaderTarget->currentText().section(" ",1,1);
      
      bootloaderTarget->clear();
      for ( QStringList::Iterator it = interfacesList.begin(); it != interfacesList.end(); ++it ) {
          QString line = *it;
          QString device = line.section(":",0,0);
          QString type = line.section(":",1,1);
          QString size = line.section(":",2,2);
          size = this->formatSizeLong(size.toLongLong());
          bootloaderTarget->insertItem( DesktopIcon("hdd_unmount"), QString("%1: %2 (%3)").arg(type).arg(device).arg(size));
//           bootloaderTarget->insertItem( DesktopIcon("hdd_unmount"), QString("%1 ").arg(line));
      }
      bootloaderTarget->setCurrentItem(0);
      for( int i = 0; i < bootloaderTarget->count(); i++)
      {
         if(bootloaderTarget->text(i).section(" ",1,1) != oldBootloaderTarget) continue;
         bootloaderTarget->setCurrentItem(i);
         break;
      }
//       bootloaderTarget->setCurrentItem(bootloaderTarget->listBox()->findItem());
  }
  
  /********************************** Device Map **********************************/
  else if(this->currentPage() == wpBootOrder)
  {
      backend->exec("list_device_map");
      backend->wait();
      
      QStringList deviceMapList = QStringList::split( "\n", backend->getOutput() );
      
      deviceMapNumber->clear();
      deviceMapDevice->clear();
      for ( QStringList::Iterator it = deviceMapList.begin(); it != deviceMapList.end(); ++it ) {
          QString line = *it;
          QString number = line.section("\t",0,0);
          QString device = line.section("\t",1,1);
          QString size = line.section("\t",2,2);
          size = this->formatSizeLong(size.toLongLong());
          QString model = line.section("\t",3,3);
          deviceMapNumber->insertItem(number);
          deviceMapDevice->insertItem(QString("%1 %2 (%3)").arg(device).arg(model).arg(size));
      }
  }
  
  /********************************** Root-account configuration **********************************/
  else if(this->currentPage() == wpRoot)
  {
    
  }
  
  /********************************** User configuration **********************************/
  else if(this->currentPage() == wpUser)
  {
    
  }
  
  /********************************** Confirm Installation **********************************/
  else if(this->currentPage() == wpConfirmInstallation)
  {
    confirmInstallationList->clear();
    if(installTypeList->currentItem().id() == 1 || installTypeList->currentItem().id() == 2)
    {
      if(installTypeList->currentItem().id() == 2) confirmInstallationList->insertItem(AcriListInfo::AcriListInfo(0,i18n("Installation type"),i18n("Update Install"),"acritoxinstaller"));
      backend->exec("get_config hd_map"); backend->wait();
      QStringList hdMapList = QStringList::split( "\n", backend->getOutput() );
      for ( QStringList::Iterator it = hdMapList.begin(); it != hdMapList.end(); ++it ) {
          QString line = *it;
          QString device = line.section(":",0,0);
          QString mountpoint = line.section(":",1,1);
          QString filesystem = line.section(":",2,2);
          QString automount = line.section(":",3,3);
          QString partition_name = i18n("%1 will be used as %2").arg(device).arg(mountpoint);
          QString partition_description;
          if(filesystem) partition_description = i18n("It will be formatted with %1").arg(filesystem);
          if(automount == "auto") partition_description += QString("\n") + i18n("It will be mounted automatically on boot.");
          confirmInstallationList->insertItem(AcriListInfo::AcriListInfo(0,partition_name,partition_description,"hdd_unmount"));
      }
      
      backend->exec("get_config bootloader"); backend->wait();
      QString bootloader = backend->getOutput().stripWhiteSpace();
      backend->exec("get_config bootloadertarget"); backend->wait();
      QString bootloadertarget = backend->getOutput().stripWhiteSpace();
      QString bootloader_description;
      if(bootloader == "grub")
        bootloader_description = i18n("%1 will be installed to %2").arg("GRUB").arg(bootloadertarget);
      else if(bootloader == "lilo")
        bootloader_description = i18n("%1 will be installed to %2").arg("Lilo").arg(bootloadertarget);
      confirmInstallationList->insertItem(AcriListInfo::AcriListInfo(0,i18n("Bootloader"),bootloader_description,"run"));
    }
  }
  
  /********************************** Installation **********************************/
  else if(this->currentPage() == wpInstallation)
  {
    cancelButton()->setEnabled(false);
    setStatusBusy(true); // Don't allow the user to close the window now...
    switch(installTypeList->currentItem().id())
    {
    case 1:
      doInstall();
      break;
    case 2:
      doUpdateInstall();
      break;
    case 3:
      doRecoverInstall();
      break;
    }
  }
  QApplication::restoreOverrideCursor();
}

void AcritoxInstaller::doInstall()
{
    connect(backend, SIGNAL(receivingStdout()),
            this, SLOT(installLogReceivedOutput()));
    connect(backend, SIGNAL(receivedProgress(int)),
            progressCurrent, SLOT(setProgress(int)));
    connect(&timer,SIGNAL(timeout()),this,SLOT(updateProgress()));
    timer.start(25);
    
    installLog->append(QString("* ") + i18n("Installation started..."));
    progressCompleted->setTotalSteps(100);
    
    installLog->append(QString("* ") + i18n("Checking partitions..."));
    progressCurrent->setTotalSteps(100);
    progressCurrent->setProgress(0);
    backend->exec("check_partitions_for_install");
    progressCompleted->setProgress(3);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Preparing partitions"));
    backend->exec("prepare_partitions_for_install");
    progressCompleted->setProgress(10);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Copying system... Please wait..."));
    backend->exec("copy_system_to_target");
    progressCompleted->setProgress(60);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Updating /etc/fstab..."));
    backend->exec("update_fstab_on_target");
    progressCompleted->setProgress(65);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setTotalSteps(0);
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Updating /etc/passwd..."));
    backend->exec("update_passwd_on_target");
    progressCompleted->setProgress(67);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Copying /home..."));
    backend->exec("copy_home_to_target");
    progressCompleted->setProgress(70);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Copying /etc..."));
    backend->exec("copy_etc_to_target");
    progressCompleted->setProgress(72);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Configuring..."));
    backend->exec("configure_target_update_files");
    progressCompleted->setProgress(74);
    if(backend->errorOccurred()) return;
    backend->exec("configure_target_keymap");
    progressCompleted->setProgress(75);
    if(backend->errorOccurred()) return;
    backend->exec("configure_target_purge_live_only_stuff");
    progressCompleted->setProgress(80);
    if(backend->errorOccurred()) return;
    backend->exec("configure_target_modules");
    progressCompleted->setProgress(82);
    if(backend->errorOccurred()) return;
    progressCurrent->setTotalSteps(100);
    backend->exec("configure_target_services");
    progressCompleted->setProgress(90);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setTotalSteps(0);
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Installing bootmanager..."));
    backend->exec("install_bootmanager_to_target");
    progressCompleted->setProgress(99);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Cleanup..."));
//     backend->exec("replace_devices_on_target");
    backend->exec("cleanup");
    progressCompleted->setProgress(100);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setTotalSteps(100);
    progressCurrent->setProgress(100);
    
    installLog->append(QString("* ") + i18n("Installation finished!"));
    QApplication::restoreOverrideCursor();
    setFinishEnabled(wpInstallation, true);
    statusFinished = true;
    setStatusBusy(false);
    this->cancelButton()->setDisabled(true);
    
    disconnect(&timer,SIGNAL(timeout()),this,SLOT(updateProgress()));
    disconnect(backend, SIGNAL(receivedProgress(int)),
            progressCurrent, SLOT(setProgress(int)));
    disconnect(backend, SIGNAL(receivingStdout()),
            this, SLOT(installLogReceivedOutput()));
}

void AcritoxInstaller::doUpdateInstall()
{
    connect(backend, SIGNAL(receivingStdout()),
            this, SLOT(installLogReceivedOutput()));
    connect(backend, SIGNAL(receivedProgress(int)),
            progressCurrent, SLOT(setProgress(int)));
    connect(&timer,SIGNAL(timeout()),this,SLOT(updateProgress()));
    timer.start(25);
    
    installLog->append(QString("* ") + i18n("Update Installation started..."));
    progressCompleted->setTotalSteps(100);
    
    installLog->append(QString("* ") + i18n("Preparing partitions"));
    progressCurrent->setTotalSteps(100);
    progressCurrent->setProgress(0);
    backend->exec("prepare_partitions_for_install --nomount");
    progressCompleted->setProgress(2);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Update Install Prepatch..."));
    backend->exec("update_install_prepatch");
    progressCompleted->setProgress(5);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Deleting files... Please wait..."));
    backend->exec("update_install_delete");
    progressCompleted->setProgress(10);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Moving files... Please wait..."));
    backend->exec("update_install_move");
    progressCompleted->setProgress(30);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setTotalSteps(100);
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Copying system... Please wait..."));
    backend->exec("copy_system_to_target");
    progressCompleted->setProgress(80);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Updating /etc/fstab..."));
    backend->exec("update_fstab_on_target");
    progressCompleted->setProgress(82);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setTotalSteps(0);
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Copying /etc..."));
    backend->exec("copy_etc_to_target");
    progressCompleted->setProgress(83);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Configuring..."));
    backend->exec("configure_target_update_files");
    progressCompleted->setProgress(84);
    if(backend->errorOccurred()) return;
    backend->exec("configure_target_keymap");
    progressCompleted->setProgress(85);
    if(backend->errorOccurred()) return;
    backend->exec("configure_target_purge_live_only_stuff");
    progressCompleted->setProgress(87);
    if(backend->errorOccurred()) return;
    backend->exec("configure_target_modules");
    progressCompleted->setProgress(88);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setTotalSteps(100);
    backend->exec("configure_target_services");
    progressCompleted->setProgress(91);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setTotalSteps(0);
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Installing bootmanager..."));
    backend->exec("install_bootmanager_to_target");
    progressCompleted->setProgress(95);
    if(backend->errorOccurred()) return;
    
    installLog->append(QString("* ") + i18n("Moving files back... Please wait..."));
    backend->exec("update_install_move_back");
    progressCompleted->setProgress(99);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Cleanup..."));
//     backend->exec("replace_devices_on_target");
    backend->exec("cleanup");
    progressCompleted->setProgress(100);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setTotalSteps(100);
    progressCurrent->setProgress(100);
    
    installLog->append(QString("* ") + i18n("Installation finished!"));
    QApplication::restoreOverrideCursor();
    setFinishEnabled(wpInstallation, true);
    statusFinished = true;
    setStatusBusy(false);
    this->cancelButton()->setDisabled(true);
    
    disconnect(&timer,SIGNAL(timeout()),this,SLOT(updateProgress()));
    disconnect(backend, SIGNAL(receivedProgress(int)),
            progressCurrent, SLOT(setProgress(int)));
    disconnect(backend, SIGNAL(receivingStdout()),
            this, SLOT(installLogReceivedOutput()));
}


void AcritoxInstaller::doRecoverInstall()
{
    connect(backend, SIGNAL(receivingStdout()),
            this, SLOT(installLogReceivedOutput()));
    connect(backend, SIGNAL(receivedProgress(int)),
            progressCurrent, SLOT(setProgress(int)));
    connect(&timer,SIGNAL(timeout()),this,SLOT(updateProgress()));
    timer.start(25);
    
    installLog->append(QString("* ") + i18n("Recover started..."));
    progressCompleted->setTotalSteps(100);
    
    progressCurrent->setTotalSteps(0);
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Installing bootmanager..."));
    backend->exec("install_grub");
    progressCompleted->setProgress(95);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setProgress(0);
    installLog->append(QString("* ") + i18n("Cleanup..."));
//     backend->exec("replace_devices_on_target");
    backend->exec("recover_cleanup");
    progressCompleted->setProgress(100);
    if(backend->errorOccurred()) return;
    
    progressCurrent->setTotalSteps(100);
    progressCurrent->setProgress(100);
    
    installLog->append(QString("* ") + i18n("Recover finished!"));
    QApplication::restoreOverrideCursor();
    setFinishEnabled(wpInstallation, true);
    statusFinished = true;
    setStatusBusy(false);
    this->cancelButton()->setDisabled(true);
    
    disconnect(&timer,SIGNAL(timeout()),this,SLOT(updateProgress()));
    disconnect(backend, SIGNAL(receivedProgress(int)),
            progressCurrent, SLOT(setProgress(int)));
    disconnect(backend, SIGNAL(receivingStdout()),
            this, SLOT(installLogReceivedOutput()));
}

void AcritoxInstaller::updateProgress()
{
  if(progressCurrent->totalSteps() == 0)
  {
    int currentPos = progressCurrent->progress();
    progressCurrent->setPercentageVisible(false);
    progressCurrent->setProgress(++currentPos);
  }
  else
  {
    progressCurrent->setPercentageVisible(true);
    
    if(installTypeList->currentItem().id() == 2) // Update install
    {
      if(progressCompleted->progress() >= 30 and progressCompleted->progress() < 80)
      {
        // copying system to target...
        progressCompleted->setProgress(30+progressCurrent->progress()/2);
      }
    }
    else
    {
      if(progressCompleted->progress() >= 10 and progressCompleted->progress() < 60)
      {
        // copying system to target...
        progressCompleted->setProgress(10+progressCurrent->progress()/2);
      }
    }
  }
}

void AcritoxInstaller::prepareLogReceivedOutput()
{
  partitionsQueryLog->append(backend->getOutput());
  partitionsQueryLog->repaint();
}

void AcritoxInstaller::runLogReceivedOutput()
{
  partitionsRunLog->append(backend->getOutput());
  partitionsRunLog->repaint();
}

void AcritoxInstaller::installLogReceivedOutput()
{
  installLog->append(backend->getOutput());
  installLog->repaint();
}

void AcritoxInstaller::handleBackendError(uint errorcode, QString error)
{
    progressEnd();
    QApplication::restoreOverrideCursor();
    QMessageBox::critical( 0, i18n("Backend Error"),
        i18n("An internal error occurred: Error %1\n\n%2").arg(errorcode).arg(error) );
}

void AcritoxInstaller::progressEnd()
{
    delete progressDlg;
    progressDlg = 0;
}

void AcritoxInstaller::realnameChanged(const QString &name)
{
    if(username->text().length()) return;
    if(name.contains(" ")) username->setText(name.section(" ",0,0).lower());
}

QString AcritoxInstaller::formatSizeLong( Q_ULLONG size )
{
  QString sizeText;
  int count = 0;
   
  while ( size > 0 )
  {
    sizeText = ( ( size % 10 ) + '0' ) + sizeText;
    size /= 10;
     
    if ( ++count == 3 && size > 0 )
    {
      sizeText = KGlobal::locale()->thousandsSeparator() + sizeText;
      count = 0;
    }
  }
   
  return sizeText;
}

QString AcritoxInstaller::encrypt( QString pwd )
{
    unsigned long seed[2];
    char salt[] = "$1$........";
    const char *const seedchars =
        "./0123456789ABCDEFGHIJKLMNOPQRST"
        "UVWXYZabcdefghijklmnopqrstuvwxyz";
    int i;
    
    seed[0] = KApplication::random();
    seed[1] = KApplication::random() ^ (seed[0] >> 14 & 0x30000);
    
    for (i = 0; i < 8; i++)
        salt[3+i] = seedchars[(seed[i/5] >> (i%5)*6) & 0x3f];
    
    QString password = crypt(pwd, salt);
    
    password.replace(QChar('$'),"\\$");
    
    return( password );
}

void AcritoxInstaller::confirmStateChanged()
{
  setNextEnabled(wpConfirmPartitioning, confirmCheckBox->isChecked());
  setBackEnabled(wpConfirmPartitioning, !confirmCheckBox->isChecked());
}

void AcritoxInstaller::confirmInstallStateChanged()
{
  setNextEnabled(wpConfirmInstallation, confirmInstallCheckBox->isChecked());
  setBackEnabled(wpConfirmInstallation, !confirmInstallCheckBox->isChecked());
}

void AcritoxInstaller::updateRootPasswordStatus()
{
  QString rootPW1 = rootPassword1->password();
  QString rootPW2 = rootPassword2->password();
  
  rootPassword1Status->setPixmap(SmallIcon("button_ok"));
  rootPassword2Status->setPixmap(SmallIcon("button_ok"));
  QToolTip::remove(rootPassword1Status);
  QToolTip::remove(rootPassword2Status);
  
  bool passwordsOk = true;
  
  if(!rootPW1.length())
  {
    QToolTip::add(rootPassword1Status, i18n("Please enter a password!"));
    rootPassword1Status->setPixmap(SmallIcon("messagebox_warning"));
    passwordsOk = false;
  }
  else if(rootPW1.length() < 6)
  {
    QToolTip::add(rootPassword1Status, i18n("Password must have at least 6 characters!"));
    rootPassword1Status->setPixmap(SmallIcon("messagebox_warning"));
    passwordsOk = false;
  }
  
  if(!rootPW2.length())
  {
    QToolTip::add(rootPassword2Status, i18n("Please enter a password!"));
    rootPassword2Status->setPixmap(SmallIcon("messagebox_warning"));
    passwordsOk = false;
  }
  else if(rootPW2.length() < 6)
  {
    QToolTip::add(rootPassword2Status, i18n("Password must have at least 6 characters!"));
    rootPassword2Status->setPixmap(SmallIcon("messagebox_warning"));
    passwordsOk = false;
  }
  
  if(passwordsOk)
  if(rootPW1 != rootPW2)
  {
    rootPassword1Status->setPixmap(SmallIcon("messagebox_warning"));
    rootPassword2Status->setPixmap(SmallIcon("messagebox_warning"));
    QToolTip::add(rootPassword1Status, i18n("Passwords don't match!"));
    QToolTip::add(rootPassword2Status, i18n("Passwords don't match!"));
  }
  else
  {
    setNextEnabled(wpRoot, true);
  }
}

void AcritoxInstaller::updateUserPasswordStatus()
{
  QString userPW1 = userPassword1->password();
  QString userPW2 = userPassword2->password();
  
  userPassword1Status->setPixmap(SmallIcon("button_ok"));
  userPassword2Status->setPixmap(SmallIcon("button_ok"));
  QToolTip::remove(userPassword1Status);
  QToolTip::remove(userPassword2Status);
  
  bool passwordsOk = true;
  
  if(!userPW1.length())
  {
    QToolTip::add(userPassword1Status, i18n("Please enter a password!"));
    userPassword1Status->setPixmap(SmallIcon("messagebox_warning"));
    passwordsOk = false;
  }
  else if(userPW1.length() < 6)
  {
    QToolTip::add(userPassword1Status, i18n("Password must have at least 6 characters!"));
    userPassword1Status->setPixmap(SmallIcon("messagebox_warning"));
    passwordsOk = false;
  }
  
  if(!userPW2.length())
  {
    QToolTip::add(userPassword2Status, i18n("Please enter a password!"));
    userPassword2Status->setPixmap(SmallIcon("messagebox_warning"));
    passwordsOk = false;
  }
  else if(userPW2.length() < 6)
  {
    QToolTip::add(userPassword2Status, i18n("Password must have at least 6 characters!"));
    userPassword2Status->setPixmap(SmallIcon("messagebox_warning"));
    passwordsOk = false;
  }
  
  if(passwordsOk)
  if(userPW1 != userPW2)
  {
    userPassword1Status->setPixmap(SmallIcon("messagebox_warning"));
    userPassword2Status->setPixmap(SmallIcon("messagebox_warning"));
    QToolTip::add(userPassword1Status, i18n("Passwords don't match!"));
    QToolTip::add(userPassword2Status, i18n("Passwords don't match!"));
  }
  else
  {
    setNextEnabled(wpUser, true);
  }
}

void AcritoxInstaller::deviceMapSetButtonsEnabled()
{
  deviceMapUp->setEnabled(deviceMapDevice->currentItem() > 0);
  deviceMapDown->setEnabled(deviceMapDevice->currentItem() > -1 && deviceMapDevice->currentItem() < (int) deviceMapDevice->count() - 1);
}

void AcritoxInstaller::deviceMapButtonDownClicked()
{
  int c = deviceMapDevice->currentItem();
  if ( c < 0 || c == int( deviceMapDevice->count() ) - 1 ) return;
  QListBoxItem *item = deviceMapDevice->item( c );
  deviceMapDevice->takeItem( item );
  deviceMapDevice->insertItem( item, c+1 );
  deviceMapDevice->setCurrentItem( item );
}

void AcritoxInstaller::deviceMapButtonUpClicked()
{
  int c = deviceMapDevice->currentItem();
  if ( c < 1 ) return;
  QListBoxItem *item = deviceMapDevice->item( c );
  deviceMapDevice->takeItem( item );
  deviceMapDevice->insertItem( item, c-1 );
  deviceMapDevice->setCurrentItem( item );
}

void AcritoxInstaller::confirmSwapStateChanged()
{
  setNextEnabled(wpSwap, confirmSwapAdjustments->isChecked());
}


#include "acritoxinstaller.moc"

