/***************************************************************************
 *   Copyright (C) 2007 by Andreas Loibl   *
 *   andreas@andreas-loibl.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qapplication.h>
#include <qeventloop.h>
#include <qmap.h>

#include "backend.h"

Backend::Backend()
{
    _process = new KProcIO();
    *_process << "/usr/share/acritoxinstaller/backend.sh" << "-ni";
    
    connect(_process, SIGNAL(processExited(KProcess*)), this, SLOT(slotProcessExited()));
    connect(_process, SIGNAL(readReady(KProcIO*)), this, SLOT(gotOutput()));
    
    exec_done = true;
    query_received = true;
    error_received = false;
    exited = false;
    debug = false;
}

Backend::~Backend()
{}

void Backend::runBackend()
{
    _process->start(KProcess::NotifyOnExit, true);
}

void Backend::slotProcessExited()
{
  exited=true;
  emit processExited();
}

void Backend::gotOutput()
{
  emit receivingStdout();
  QString line;
  while(true) {
     int result = _process->readln(line);
     if (result == -1) break;
     
     if (line.contains("<acritoxinstaller debug>"))
     {
       QString dataline = line.replace("<acritoxinstaller debug>", "+");
       qDebug("DBG: " + dataline);
     }
     else
     {
       qDebug("< "+line);
       
       if (line.startsWith("<acritoxinstaller backend error>"))
       {
          QString dataline = line.mid(QString("<acritoxinstaller backend error>").length()+1);
          uint errorcode = dataline.section(" ",0,0).toInt();
          QString description = dataline.section(" ",1);
          error_received = true;
          qDebug( "Error " + QString::number(errorcode) + ": " + description );
          emit receivedError( errorcode, description );
       }
       else if (line.startsWith("<acritoxinstaller backend prompt>"))
       {
          query_received = false;
          exec_done = true;
          emit receivedStdout();
       }
       else if (line.startsWith("<acritoxinstaller backend data>"))
       {
          QString dataline = line.mid(QString("<acritoxinstaller backend data>").length()+1);
          QString var = dataline.section(" ",0,0);
          QString value = dataline.section(" ",1);
          _data[var] = value;
       }
       else if (line.startsWith("<acritoxinstaller backend query>"))
       {
          _query = line.mid(QString("<acritoxinstaller backend query>").length());
          query_received = true;
          emit receivedQuery();
       }
       else if (line.startsWith("<acritoxinstaller backend progress>"))
       {
          emit receivedProgress(line.mid(QString("<acritoxinstaller backend progress>").length()+1).toInt());
       }
       else
       {
       _buffer.append(line).append("\n");
       }
     }
  }
}

QString Backend::getOutput()
{
  QString buffer = _buffer;
  _buffer = "";
  return buffer;
}

QString Backend::getQuery()
{
  QString query = _query;
  _query = "";
  return query;
}

void Backend::answerQuery(QString answer)
{
  writeStdin( answer );
  _query = "";
  query_received = false;
  wait();
}

bool Backend::writeStdin(QString data)
{
  qDebug("> "+data);
  return _process->writeStdin(data);
}

bool Backend::exec(QString command, bool dowait)
{
  wait();
  exec_done = false;
  query_received = false;
  error_received = false;
  _buffer = "";
  // _process->writeStdin(command.append("\n"));
  writeStdin(command);
  if(dowait) return wait();
}

bool Backend::wait()
{
  if(exec_done) return false;
  while(!query_received && !exec_done && !exited) {QApplication::eventLoop()->processEvents( QEventLoop::ExcludeUserInput );}
  return query_received;
}

bool Backend::isQuery()
{
  return query_received;
}

bool Backend::errorOccurred()
{
  return error_received;
}

QString Backend::data(QString var)
{
  return _data[var];
}
