/***************************************************************************
 *   Copyright (C) 2007 by Andreas Loibl   *
 *   andreas@andreas-loibl.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef BACKEND_H
#define BACKEND_H

#include <qmap.h>
#include <kprocio.h>

class Backend : public QObject
{
    Q_OBJECT

public:
    Backend();
    ~Backend();
    void runBackend();
    bool writeStdin(QString data);
    QString getOutput();
    QString getQuery();
    void answerQuery(QString answer);
    bool wait();
    bool isQuery();
    bool errorOccurred();
    bool exec(QString command, bool dowait = true);
    QString data(QString var);
    bool debug;

protected:
    KProcIO* _process;

private:
    QString _buffer;
    QString _query;
    bool exec_done;
    bool query_received;
    bool error_received;
    bool exited;
    typedef QMap<QString, QString> DataTable;
    DataTable _data;

private slots:
    void gotOutput();
    void slotProcessExited();

signals:
    void receivingStdout();
    void receivedStdout();
    void receivedQuery();
    void receivedError(uint errorcode, QString error);
    void receivedProgress(int percent);
    void processExited();
};

#endif
