#ifndef DIRSELECTOR_H
#define DIRSELECTOR_H

#include <qlistview.h>

#include <kdirlister.h>
#include <kurl.h>

class DirSelector : public QListView
{
 public:

  DirSelector(QWidget * parent = 0, const char * name = 0);
  ~DirSelector();
  
  static DirSelector*         instance() { return s_instance; }
  void                        setDirectories(const QStringList &);
  QStringList &               directories() { return fDirectories; }

  void                        setTopDir(const KURL & url);

  void                        refresh();
  
  QStringList                 pathsDisabled;
  QStringList                 pathsRDisabled; /* recursive paths */

 protected:

  QStringList                 fDirectories;
  KURL                        fTopDir;
  
 private:
  static DirSelector*       s_instance;
};

class DirSelectorItem : public QObject, public QCheckListItem
{
Q_OBJECT

 public:

  DirSelectorItem(QListView * parent, const KURL & url);
  DirSelectorItem(QListViewItem * parent, const KURL & url, bool disable, bool rdisable);

  QCheckListItem *            parent() const;
  bool                        isRDisabled() const { return fRDisabled; }
  bool                        isDisabled() const { return fDisabled || isRDisabled() || ( parent() && parent()->isOn() ); }
  KURL &                      url() { return fURL; }

  void                        setOpen(bool b);
  void                        stateChange(bool b);
  void                        activate();
  void                        paintCell(QPainter * p, const QColorGroup & cg,
					int column, int width, int align);

public slots:

  void                        newItems(const KFileItemList &);
  void                        completed();

private:

  KDirLister                  fDirLister;
  KURL                        fURL;
  bool                        fListed;
  bool                        fDisabled;
  bool                        fRDisabled;
};
#endif
